package game.renderer;

import java.nio.ByteBuffer;
import java.util.List;

import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL13;

import game.window.WCF;

public abstract class Tessellator
{
    private static final RenderBuffer BUFFER = new RenderBuffer(2097152);

    public static RenderBuffer getBuffer()
    {
        return BUFFER;
    }
    
    public static void draw()
    {
        BUFFER.finishDrawing();
        if (BUFFER.getVertexCount() > 0)
        {
            VertexFormat vertexformat = BUFFER.getVertexFormat();
            int i = vertexformat.getNextOffset();
            ByteBuffer bytebuffer = BUFFER.getByteBuffer();
            List<VertexFormatElement> list = vertexformat.getElements();

            for (int j = 0; j < list.size(); ++j)
            {
                VertexFormatElement vertexformatelement = (VertexFormatElement)list.get(j);
                VertexFormatElement.EnumUsage vertexformatelement$enumusage = vertexformatelement.getUsage();
                int k = vertexformatelement.getType().getGlConstant();
                int l = vertexformatelement.getIndex();
                bytebuffer.position(vertexformat.getOffset(j));

                switch (vertexformatelement$enumusage)
                {
                    case POSITION:
                        WCF.glVertexPointer(vertexformatelement.getElementCount(), k, i, bytebuffer);
                        WCF.glEnableClientState(GL11.GL_VERTEX_ARRAY);
                        break;

                    case UV:
					WCF.glClientActiveTexture(GL13.GL_TEXTURE0 + l);
                        WCF.glTexCoordPointer(vertexformatelement.getElementCount(), k, i, bytebuffer);
                        WCF.glEnableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
					WCF.glClientActiveTexture(GL13.GL_TEXTURE0);
                        break;

                    case COLOR:
                        WCF.glColorPointer(vertexformatelement.getElementCount(), k, i, bytebuffer);
                        WCF.glEnableClientState(GL11.GL_COLOR_ARRAY);
                        break;

                    case NORMAL:
                        WCF.glNormalPointer(k, i, bytebuffer);
                        WCF.glEnableClientState(GL11.GL_NORMAL_ARRAY);
                }
            }

            WCF.glDrawArrays(BUFFER.getDrawMode(), 0, BUFFER.getVertexCount());
            int i1 = 0;

            for (int j1 = list.size(); i1 < j1; ++i1)
            {
                VertexFormatElement vertexformatelement1 = (VertexFormatElement)list.get(i1);
                VertexFormatElement.EnumUsage vertexformatelement$enumusage1 = vertexformatelement1.getUsage();
                int k1 = vertexformatelement1.getIndex();

                switch (vertexformatelement$enumusage1)
                {
                    case POSITION:
                        WCF.glDisableClientState(GL11.GL_VERTEX_ARRAY);
                        break;

                    case UV:
					WCF.glClientActiveTexture(GL13.GL_TEXTURE0 + k1);
                        WCF.glDisableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
					WCF.glClientActiveTexture(GL13.GL_TEXTURE0);
                        break;

                    case COLOR:
                        WCF.glDisableClientState(GL11.GL_COLOR_ARRAY);
                        GlState.resetColor();
                        break;

                    case NORMAL:
                        WCF.glDisableClientState(GL11.GL_NORMAL_ARRAY);
                }
            }
        }
        BUFFER.reset();
    }
}
